package com.deryckwilliams.botControl1;

import java.io.IOException;

import com.deryckwilliams.botControl1.android.R;
import com.deryckwilliams.botControl1.arduino.BotProtocol;
import com.deryckwilliams.botControl1.arduino.Cmd;
import com.deryckwilliams.botControl1.arduino.CmdVal;

/*******************************************************************************
 */
public final class BotController {
	private BotCommsInterface myComms;
	private BotUIInterface myUI;

	private boolean inLoop = false;
    private boolean timeToGo = false;
    
	private final static int MAX_MOTOR_VAL = 255;
	private final static int MAX_SERVO_VAL = 100;
	
	/***************************************************************************
	 */
	public BotController(BotUIInterface botUI, BotCommsInterface botComms) {
		super();
		this.myUI = botUI;
		this.myComms = botComms;
	}
	
	/***************************************************************************
	 */
	public void disconnect() {
		timeToGo = true;
	}

	/***************************************************************************
	 */
	public boolean isConnected() {
		return inLoop;
	}

	/***************************************************************************
	 */
	public void processingLoop() {
		long lastBattCheck = 0;
		long lastProxCheck = 0;
		long now;
		inLoop = true;
        while (!timeToGo) {
    		try {
    			Thread.sleep(150);
    		}
    		catch(InterruptedException ex) {}

    		myComms.checkRecvMsg();
    		
    		// send the speed commands
    		if(myUI.doingClaw())
    			sendClawPos();
    		else
    			sendSpeeds();
    		
    		now = System.currentTimeMillis();
    		if((now - lastBattCheck) > 1000) {
    			sendBattReq();
    			lastBattCheck = now;
    		}
    		
    		if((now - lastProxCheck) > 150) {
    			sendProxReq();
    			lastProxCheck = now;
    		}
        }
        inLoop = false;
	}
	
	/***************************************************************************
	 */
	private void sendBattReq() {
		try {
			myComms.sendCmd(new Cmd(BotProtocol.CMDS_POWER, BotProtocol.CMD_BAT_LEVEL_REQ));
		} catch (IOException e) {
			myUI.setCommDisp(R.string.lost_connection);
			timeToGo = true;
		}

	}

	/***************************************************************************
	 */
	private void sendProxReq() {
		try {
			myComms.sendCmd(new Cmd(BotProtocol.CMDS_OTHER_SENS, BotProtocol.CMD_PROX_SENS_REQ));
		} catch (IOException e) {
			myUI.setCommDisp(R.string.lost_connection);
			timeToGo = true;
		}

	}

	/***************************************************************************
	 */
	private void sendSpeeds()
	{
		int f = myUI.getFwdVal();
		int l = myUI.getLeftVal();

		int leftM, rightM;
		leftM = f - l;
		rightM = f + l;
		
		int adj = 0;
		if(leftM > 100)
		{
		    adj = leftM - 100;
		}
		if(leftM < -100)
		{
			adj = leftM + 100;
		}
		if(rightM > 100)
		{
			adj = rightM - 100;
		}
		if(rightM < -100)
		{
			adj = rightM + 100;
		}
		leftM -= adj;
		rightM -= adj;
		
		try {
			CmdVal tmp;
			
			if(leftM < 0)
				tmp = new CmdVal(BotProtocol.CMDS_DRIVE, BotProtocol.CMD_LEFT_REV, Math.abs(leftM * MAX_MOTOR_VAL) / 100);
			else
				tmp = new CmdVal(BotProtocol.CMDS_DRIVE, BotProtocol.CMD_LEFT, Math.abs(leftM * MAX_MOTOR_VAL) / 100);
			myComms.sendCmd(tmp);

			if(rightM < 0)
				tmp = new CmdVal(BotProtocol.CMDS_DRIVE, BotProtocol.CMD_RIGHT_REV, Math.abs(rightM * MAX_MOTOR_VAL) / 100);
			else
				tmp = new CmdVal(BotProtocol.CMDS_DRIVE, BotProtocol.CMD_RIGHT, Math.abs(rightM * MAX_MOTOR_VAL) / 100);
			myComms.sendCmd(tmp);
		} catch (IOException e) {}
	}

	/***************************************************************************
	 */
	private void sendClawPos()
	{
		int f = myUI.getFwdVal();
		int l = myUI.getLeftVal();

		if(f > 100) f = 100;
		if(f < -100) f = -100;
		if(l > 100) l = 100;
		if(l < -100) l = -100;
		
		try {
			CmdVal tmp;
			
			if(l < 0)
				tmp = new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_PAN_NEG, Math.abs(l * MAX_SERVO_VAL) / 100);
			else
				tmp = new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_PAN, Math.abs(l * MAX_SERVO_VAL) / 100);
			myComms.sendCmd(tmp);

			if(f < 0)
				tmp = new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_TILT, Math.abs(f * MAX_SERVO_VAL) / 100);
			else
				tmp = new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_TILT_NEG, Math.abs(f * MAX_SERVO_VAL) / 100);
			myComms.sendCmd(tmp);
		} catch (IOException e) {}
	}
	
	/*********************************************************************************
	 */
	public void recvMsg(CmdVal cmdVal) {
		  BotBattery batt;

		  switch(cmdVal.chCmd)
		  {
		  case BotProtocol.CMDS_POWER:
			  switch(cmdVal.chSubCmd)
			  {
			  case BotProtocol.CMD_BAT_LEVEL_RESP:
			    batt = new BotBattery(cmdVal.iVal);
			    myUI.setBattery(batt);
			    break;
			  }
			  break;
			  
		  case BotProtocol.CMDS_OTHER_SENS:
			  switch(cmdVal.chSubCmd)
			  {
			  case BotProtocol.CMD_PROX_SENS_RESP:
				  if(cmdVal.iVal != 0) {
					  myUI.showProxWarning();
				  }
				  break;
			  }
			  break;
		  }
	}
}
