package com.deryckwilliams.botControl1.android;

import java.io.IOException;
import java.text.NumberFormat;

import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.Vibrator;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.deryckwilliams.botControl1.BotBattery;
import com.deryckwilliams.botControl1.BotUIInterface;
import com.deryckwilliams.botControl1.arduino.BotProtocol;
import com.deryckwilliams.botControl1.arduino.Cmd;
import com.deryckwilliams.botControl1.arduino.CmdVal;

/*******************************************************************************
 */
public final class BotCtrlUI extends Activity implements SensorEventListener, BotUIInterface {

	private Sensor mAccel;
	private SensorManager mSensorManager;
	private MyBlueConnection myBt;
    private PowerManager.WakeLock wl;
	
	private int leftVal, fwdVal;
	private boolean going;
	private Button goHold;
	private boolean claw;
	private Button wheelClaw;

	private int commDisp = R.string.no_display;
	private BotBattery battery;
	private String debug = null;
	private boolean showingDialog = false;

	private static final int CONNECT_ID = Menu.FIRST;
	private static final int SETTINGS_ID = CONNECT_ID + 1;

	/***************************************************************************
	 * Called when the activity is first created.
	 */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
		// give app time to start up
		try
		{
			Thread.sleep(500);
		}
		catch(InterruptedException ex) {}

        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wl = pm.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK, "MyBotCtrl");

        mSensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        mAccel = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        
		// set up the buttons
        leftVal = fwdVal = 0;
        going = false;
        goHold = (Button)findViewById(R.id.but11);
        goHold.setText(R.string.butGo);
        goHold.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	if(going) {
                	try {
                		myBt.sendCmd(new Cmd(BotProtocol.CMDS_MODE, BotProtocol.CMD_SET_PATH));
    				} catch (IOException e)	{
    					return;
    				}
                    goHold.setText(R.string.butGo);
            		going = false;
            	} else {
                	try {
                		myBt.sendCmd(new Cmd(BotProtocol.CMDS_MODE, BotProtocol.CMD_REMOTE_CONTROL));
    				} catch (IOException e)	{
    					return;
    				}
                    goHold.setText(R.string.butHold);
            		going = true;
            	}
            }
        });

        claw = false;
        wheelClaw = (Button)findViewById(R.id.but12);
        wheelClaw.setText(R.string.butClaw);
        wheelClaw.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	if(claw) {
    				wheelClaw.setText(R.string.butClaw);
    				claw = false;
            	} else {
    				wheelClaw.setText(R.string.butWheels);
    				claw = true;
            	}
            }
        });
        
        Button butOpnFull = (Button) findViewById(R.id.but21);
        butOpnFull.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	try {
            		myBt.sendCmd(new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_OPEN, 100));
				} catch (IOException e) {}
            }
        });
    	
        Button butOpn = (Button) findViewById(R.id.but22);
        butOpn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	try {
            		myBt.sendCmd(new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_OPEN_PLUS, 20));
				} catch (IOException e) {}
            }
        });
    	
        Button butCls = (Button) findViewById(R.id.but23);
        butCls.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	try {
            		myBt.sendCmd(new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_OPEN_NEG, 20));
				} catch (IOException e) {}
            }
        });
    	
        Button butClsFull = (Button) findViewById(R.id.but24);
        butClsFull.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
            	try {
            		myBt.sendCmd(new CmdVal(BotProtocol.CMDS_CLAW, BotProtocol.CMD_CLAW_OPEN, 0));
				} catch (IOException e) {}
            }
        });
    	
        doDisplay();
    }
    
	/***************************************************************************
	 */
    private void cleanup() {
    	if(myBt != null) {
    		myBt.disconnect();
    		myBt = null;
    	}
    }
    
	/***************************************************************************
	 */
	public void connectionDone() {
    	if(showingDialog) {
	    	dismissDialog(0);
	    	removeDialog(0);
	    	showingDialog = false;
    	}
    	doDisplay();
    }
    
	/***************************************************************************
	 */
    @Override
    protected void onResume() {
    	super.onResume();
    	
    	doDisplay();

        if(mAccel != null) {
        	mSensorManager.registerListener(this, mAccel, SensorManager.SENSOR_DELAY_UI);
        }

        wl.acquire();

    }
    
	/***************************************************************************
	 */
    @Override
    protected void onPause() {
        super.onPause();

        mSensorManager.unregisterListener(this);
        wl.release();
    }
    
    /***************************************************************************
	 */
	public void setCommDisp(int disp) {
    	commDisp = disp;
    }

    /***************************************************************************
	 */
	public void debugMsg(String msg) {
    	debug = msg;
    }

    /***************************************************************************
	 */
	public void setBattery(BotBattery bat) {
    	battery = bat;
    }

    /***************************************************************************
	 */
    public void showProxWarning() {
    	Vibrator vibrator = (Vibrator)getSystemService(VIBRATOR_SERVICE);
    	vibrator.vibrate(100);
    }
    
    /***************************************************************************
	 */
    private void doDisplay() {
    	TextView leftText = (TextView)findViewById(R.id.textLeft);
    	TextView rightText = (TextView)findViewById(R.id.textRight);
    	TextView msgText = (TextView)findViewById(R.id.textMessage);
    	TextView battLevel = (TextView)findViewById(R.id.battLevel);
    	
    	leftText.setText(Integer.toString(leftVal) + "%");
    	rightText.setText(Integer.toString(fwdVal) + "%");
    	if(debug != null) {
        	msgText.setText(debug);
    	} else {
        	msgText.setText(commDisp);
    	}

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        String devicePref = prefs.getString("battDisp", "Percentage");
        if(devicePref.compareTo("Volts") == 0) {
	    	if(battery == null) {
	        	battLevel.setText("Batt -.--V");
	    	} else {
	    		NumberFormat numFmt = NumberFormat.getInstance();
	    		numFmt.setMinimumFractionDigits(2);
	    		numFmt.setMaximumFractionDigits(2);
	    		battLevel.setText("Batt " + numFmt.format(battery.getTotalVolts()) + "V");
	    	}
        } else {
	    	if(battery == null)
	        	battLevel.setText("Batt ---%");
	    	else
	    		battLevel.setText("Batt " + battery.getPctFull() + "%");
        }
    }

	/***************************************************************************
	 */
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		cleanup();
		System.exit(0);
	}

	/***************************************************************************
	 */
	@Override
	protected void onStop() {
		super.onStop();
//TODO think about options here
//		cleanup();
//		System.exit(0);
	}

	/***************************************************************************
	 */
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}

	/***************************************************************************
	 */
	public void onSensorChanged(SensorEvent event) {
		float x, y, z;
		int forward, left;
        switch(event.sensor.getType()) {
        case Sensor.TYPE_ACCELEROMETER:
        	x = event.values[0] / SensorManager.GRAVITY_EARTH;;
        	y = event.values[1] / SensorManager.GRAVITY_EARTH;;
        	z = event.values[2] / SensorManager.GRAVITY_EARTH;;
        	
        	if(z < 0)
        	{
        		if(x > 0)
        			x = 1;
        		else
        			y = 1;
        	}

        	if(x > 1) x = 1;
        	if(x < 0) x = 0;
        	if(y > 1) y = 1;
        	if(y < -1) y = -1;
        	
        	if(x < 0.4) {
        		forward = 100 - (int)(x * 100.0 / 0.4); 
        	} else if(x <= 0.6) {
        		forward = 0;
        	} else {
        		forward = - (int)((x - 0.6) * 100 / 0.4);
        	}
        	
        	if(y < -0.1) {
        		left = -(int)((y + 0.1) * 100.0 / 0.9); 
        	} else if(y <= 0.1) {
        		left = 0;
        	} else {
        		left = -(int)((y - 0.1) * 100 / 0.9);
        	}
        	
        	leftVal = left;
        	fwdVal = forward;
        	doDisplay();

        	break;
        }
	}

	/***************************************************************************
	 */
	@Override
	protected Dialog onCreateDialog(int id) {
		showingDialog = true;
		ProgressDialog diag = new ProgressDialog(this);
		diag.setMessage(getText(R.string.connecting_dlg));
		diag.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		return diag;
	}

	/***************************************************************************
	 */
	public int getLeftVal() {
		if(going)
			return leftVal;
		else
			return 0;
	}

    /***************************************************************************
	 */
	public int getFwdVal() {
		if(going)
			return fwdVal;
		else
			return 0;
	}
	
    /***************************************************************************
	 */
	public boolean doingClaw() {
		return claw;
	}

    /***************************************************************************
	 */
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        if(myBt != null && myBt.isConnected()) {
            menu.add(0, CONNECT_ID, 0, R.string.disconnect);
        } else {
        	menu.add(0, CONNECT_ID, 0, R.string.connect);
        }
        menu.add(0, SETTINGS_ID, 0, R.string.settings);
        return true;
    }
	
    /***************************************************************************
	 */
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		super.onPrepareOptionsMenu(menu);
		MenuItem item = menu.getItem(CONNECT_ID - Menu.FIRST);
        if(myBt != null && myBt.isConnected()) {
    		item.setTitle(R.string.disconnect);
        } else {
    		item.setTitle(R.string.connect);
        }
		return true;
	}

    /***************************************************************************
	 */
    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        switch(item.getItemId()) {
        case CONNECT_ID:
        	doConnect();
        	return true;
        case SETTINGS_ID:
        	showSettings();
            return true;
        }
        
        return super.onMenuItemSelected(featureId, item);
    }

    /***************************************************************************
	 */
    private void showSettings() {
        mSensorManager.unregisterListener(this);
    	Intent i = new Intent(this, BotSettings.class);
    	startActivity(i);
    }
    
    /***************************************************************************
	 */
    private void doConnect() {
		if(myBt != null && myBt.isConnected()) {
    		myBt.disconnect();
    		myBt = null;
    	} else {
    		myBt = new MyBlueConnection(this);
        	if(!myBt.btInit())
        	{
        		doDisplay();
        		return;
        	}

        	showDialog(0);
    		myBt.execute(new Object());
    	}
    }
}